/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach;

import java.util.Collection;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.nachrichten.common.grpc.NachrichtenCallContextAttachingInterceptor;
import de.ozgcloud.vorgang.common.GrpcProperty;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItem;
import de.ozgcloud.vorgang.vorgangAttachedItem.VorgangAttachedItemServiceGrpc.VorgangAttachedItemServiceBlockingStub;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class AttachedItemRemoteService {

	private static final Set<String> REPLY_ALLOWED_OPTIONS = PostfachService.REPLY_POSSIBLE_OPTION.stream().map(Enum::name)
			.collect(Collectors.toSet());
	static final String CLIENT = "OzgCloud_NachrichtenManager";
	static final String ITEM_NAME = "PostfachMail";

	@GrpcClient("vorgang-manager")
	private VorgangAttachedItemServiceBlockingStub vorgangAttachedItemServiceStub;
	@Autowired
	private PostfachNachrichtMapper postfachNachrichtMapper;

	public Stream<PostfachNachricht> findReplyAllowedPostfachNachrichts(String vorgangId) {
		return getVorgangAttachedItemServiceStub().find(buildFindRequest(vorgangId)).getVorgangAttachedItemsList().stream()
				.filter(this::replyAllowed).map(postfachNachrichtMapper::fromAttachedItem);
	}

	boolean replyAllowed(GrpcVorgangAttachedItem attachedItem) {
		return attachedItem.getItem().getPropertyList().stream().filter(this::isReplyProperty).map(GrpcProperty::getValueList)
				.flatMap(Collection::stream).anyMatch(REPLY_ALLOWED_OPTIONS::contains);
	}

	boolean isReplyProperty(GrpcProperty property) {
		return PostfachNachricht.FIELD_REPLY_OPTION.equals(property.getName());
	}

	GrpcFindVorgangAttachedItemRequest buildFindRequest(String vorgangId) {
		return GrpcFindVorgangAttachedItemRequest.newBuilder()
				.setVorgangId(vorgangId)
				.setClient(CLIENT)
				.setItemName(ITEM_NAME)
				.build();
	}

	VorgangAttachedItemServiceBlockingStub getVorgangAttachedItemServiceStub() {
		return vorgangAttachedItemServiceStub.withInterceptors(new NachrichtenCallContextAttachingInterceptor());
	}
}
