/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.attributes;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.vorgang.grpc.clientAttribute.ClientAttributeServiceGrpc.ClientAttributeServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcAccessPermission;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttributeValue;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcSetClientAttributeRequest;

class ClientAttributeRemoteServiceTest {

	@InjectMocks
	private ClientAttributeRemoteService remoteService;
	@Mock
	private ClientAttributeServiceBlockingStub stub;

	@Captor
	private ArgumentCaptor<GrpcSetClientAttributeRequest> requestCaptor;

	@BeforeEach
	void prepareStubMock() {
		when(stub.withInterceptors(any())).thenReturn(stub);
	}

	@Nested
	class TestSetBooleanReadOnlyClientAttribute {

		@Test
		void shouldCallStub() {
			callService();

			verify(stub).set(any());
		}

		@Test
		void shouldFillRequest() {
			var expectedAttribute = GrpcClientAttributeTestFactory.createBuilder()
					.setAccess(GrpcAccessPermission.READ_ONLY)
					.setValue(GrpcClientAttributeValue.newBuilder().setBoolValue(true).build())
					.build();

			var request = callService();

			assertThat(request).usingRecursiveComparison().isEqualTo(GrpcSetClientAttributeRequestTestFactory.createWithAttribute(expectedAttribute));
		}

		private GrpcSetClientAttributeRequest callService() {
			remoteService.setBooleanReadOnlyClientAttribute(GrpcSetClientAttributeRequestTestFactory.VORGANG_ID,
					ClientAttributeTestFactory.ATTRIBUTE_NAME, true);

			verify(stub).set(requestCaptor.capture());

			return requestCaptor.getValue();
		}
	}

	@Nested
	class TestSetBooleanWritableClientAttribute {

		@Test
		void shouldCallStub() {
			callService();

			verify(stub).set(any());
		}

		@Test
		void shouldFillRequest() {
			var expectedAttribute = GrpcClientAttributeTestFactory.createBuilder()
					.setAccess(GrpcAccessPermission.READ_WRITE)
					.setValue(GrpcClientAttributeValue.newBuilder().setBoolValue(true).build())
					.build();

			var request = callService();

			assertThat(request).usingRecursiveComparison().isEqualTo(GrpcSetClientAttributeRequestTestFactory.createWithAttribute(expectedAttribute));
		}

		private GrpcSetClientAttributeRequest callService() {
			remoteService.setBooleanWriteableClientAttribute(GrpcSetClientAttributeRequestTestFactory.VORGANG_ID,
					ClientAttributeTestFactory.ATTRIBUTE_NAME, true);

			verify(stub).set(requestCaptor.capture());

			return requestCaptor.getValue();
		}
	}

}
